/********************************************************************
 *	ARMˑ  
 ********************************************************************
#BASEBOARD = STM8S_D
#BASEBOARD = CQ_STARM
#BASEBOARD = STBEE
#BASEBOARD = STBEE_MINI
 *
 *
 *
 *
void Board_Init(void);			// ʔ. (0x010c)

void GPIO_Configuration(void);	// GPIO.
void led_on(void);				// LED _.
void led_off(void);				// LED .
void led_blink(int interval);	// LED _.
 *
 ********************************************************************
 */

#include "stm32f10x.h"
#include "platform_config.h"
#include "hw_config.h"
#include "usb_lib.h"
#include "usb_desc.h"
#include "usb_pwr.h"
#include "stm32_eval.h"

enum {	//HWTYPE	0:STM8S 1:STBEE 2:CQ-STARM 3:STBEE-MINI
	HW_STM8S,
	HW_STBEE,
	HW_CQ_STARM,
	HW_STBEE_MINI,
};

#define	DEFAULT_HWTYPE	HW_STM8S


enum {	//HWTYPE	0:STM8S 1:STBEE 2:CQ-STARM 3:STBEE-MINI
	_8MHz,
	_12MHz,
};

#ifdef	STM8S_D
#define	LED_PORT		GPIOA
#define	LED_PIN			GPIO_Pin_8

#define	HWTYPE	0			// 0:STM8S 1:STBEE 2:CQ-STARM 3:STBEE-MINI
#define	ROMSIZE	(64/32)		// 32kB̔{.
#define	RAMSIZE	(20/4)		//  4kB̔{.
#define	SIGNATURESTRING	"STM8S-D"

#endif

#ifdef	STBEE
#define	LED_PORT		GPIOD
#define	LED_PIN			GPIO_Pin_4
#define	USB_PULLUP_PORT	GPIOD
#define	USB_PULLUP_BIT  GPIO_Pin_3

#define	HWTYPE	1
#define	ROMSIZE	(512/32)	// 32kB̔{.
#define	RAMSIZE	(20/4)		//  4kB̔{.
#define	SIGNATURESTRING	"STBEE"

#endif

#ifdef	CQ_STARM
#define	LED_PORT		GPIOC
#define	LED_PIN			GPIO_Pin_6

#define	HWTYPE	2
#define	ROMSIZE	(128/32)	// 32kB̔{.
#define	RAMSIZE	(20/4)		//  4kB̔{.
#define	SIGNATURESTRING	"CQ-STARM"

#endif

#ifdef	STBEE_MINI
#define	LED_PORT		GPIOA
#define	LED_PIN			GPIO_Pin_15
#define	USB_PULLUP_PORT	GPIOA
#define	USB_PULLUP_BIT  GPIO_Pin_14

#define	HWTYPE	3
#define	ROMSIZE	(128/32)	// 32kB̔{.
#define	RAMSIZE	(20/4)		//  4kB̔{.
#define	SIGNATURESTRING	"STBEE-MINI"

#endif

#define	_BV(n)		( 1 << (n) )

//
//	16byte  :	@픻ʗp
//
typedef	struct {
	uchar mark[4];	// "ARM"
	uchar signid;	// 0xa3 : ARM Cortex-M3
	uchar hwtype;	// n[hEFA^Cv.
	uchar romsize;	// ROMe.
	uchar ramsize;	// RAMe.
	uchar name[12];	// ̕.
} SIGNATURE;
SIGNATURE *HW_Signature=(SIGNATURE *) 0x10c;

int	hwtype=HWTYPE;

//	=========				HW_STM8S,	HW_STBEE,	HW_CQ_STARM	,HW_STBEE_MINI,
void 	*led1_port_tab[]=	{GPIOA		,GPIOD		,GPIOC		,GPIOA,};			//LED|[g.
int		 led1_pin_tab[]=	{GPIO_Pin_8	,GPIO_Pin_4	,GPIO_Pin_6	,GPIO_Pin_13,};		//LEDsԍ.
void 	*led2_port_tab[]=	{GPIOA		,GPIOD		,GPIOC		,GPIOA,};			//LED|[g.
int		 led2_pin_tab[]=	{GPIO_Pin_8	,GPIO_Pin_4	,GPIO_Pin_6	,GPIO_Pin_15,};		//LEDsԍ.
void 	*usb_port_tab[]=	{0			,GPIOD		,0			,GPIOA,};			//USBvAbv|[g.(svȂ0)
int		 usb_pin_tab[]=		{0			,GPIO_Pin_3	,0			,GPIO_Pin_14,};		//USBvAbvsԍ.
int		 clock12M_tab[]=	{_8MHz		,_12MHz		,_8MHz		,_12MHz};			//NbN(12MHzȂ1)

/********************************************************************
 *
 ********************************************************************
 */
int	Board_Init(void)
{
	if (HW_Signature->mark[0]=='A') {
		if (HW_Signature->mark[1]=='R') {
			if (HW_Signature->mark[2]=='M') {
				if (HW_Signature->signid == 0xa3) {
					hwtype = HW_Signature->hwtype;
				}
			}
		}
	}

	return clock12M_tab[hwtype];
}

#ifdef	STBEE_MINI
//	LED _:
#define	LED_SetBits		GPIO_ResetBits
#define	LED_ResetBits	GPIO_SetBits
#else
//	LED _:
#define	LED_SetBits		GPIO_SetBits
#define	LED_ResetBits	GPIO_ResetBits
#endif
/********************************************************************
 *
 ********************************************************************
 */
void led_on(void)
{
	LED_SetBits(led1_port_tab[hwtype],led1_pin_tab[hwtype]);
}

/********************************************************************
 *
 ********************************************************************
 */
void led_off(void)
{
	LED_ResetBits(led1_port_tab[hwtype],led1_pin_tab[hwtype]);
}

/********************************************************************
 *
 ********************************************************************
 */
void led2_on(void)
{
	LED_SetBits(led2_port_tab[hwtype],led2_pin_tab[hwtype]);
}

/********************************************************************
 *
 ********************************************************************
 */
void led2_off(void)
{
	LED_ResetBits(led2_port_tab[hwtype],led2_pin_tab[hwtype]);
}

/********************************************************************
 *
 ********************************************************************
 */
void led_blink(int interval)
{
	static int led_cnt=0;
	int mask = 1<<interval;

	led_cnt++;
	if (led_cnt & mask) {
		led_on();
	} else {
		led_off();
	}
}

/********************************************************************
 *
 ********************************************************************
 */
void led2_blink(int interval)
{
	static int led_cnt=0;
	int mask = 1<<interval;

	led_cnt++;
	if (led_cnt & mask) {
		led2_on();
	} else {
		led2_off();
	}
}


/*******************************************************************************
* Function Name  : GPIO_Configuration
* Description    : Configures the different GPIO ports.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void GPIO_Configuration_STM8S_D(void)
{
	GPIO_InitTypeDef GPIO_InitStructureA;
	GPIO_InitTypeDef GPIO_InitStructureB;
	GPIO_InitTypeDef GPIO_InitStructureD;

	// GPIO A,B,C,D ɃNbN^.
	RCC_APB2PeriphClockCmd(
	    RCC_APB2Periph_GPIOA |
	    RCC_APB2Periph_GPIOB |
	    RCC_APB2Periph_GPIOC | /* == RCC_APB2Periph_GPIO_IOAIN      */
	    RCC_APB2Periph_GPIOD   /* == RCC_APB2Periph_GPIO_DISCONNECT */
	    , ENABLE);

	// GPIO_B.5 (SWIM RESET#) pull-up
	GPIO_InitStructureB.GPIO_Pin   = GPIO_Pin_5;
	GPIO_InitStructureB.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructureB.GPIO_Mode  = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOB, &GPIO_InitStructureB);

#if	BOOTLOADER_MODE
	GPIO_SetBits(GPIOB , GPIO_Pin_5);
#endif

	/* PD.09 used as USB pull-up */
	GPIO_InitStructureD.GPIO_Pin   = GPIO_Pin_9;			// PD9  USB PullUp炵.
	GPIO_InitStructureD.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructureD.GPIO_Mode  = GPIO_Mode_Out_OD;
	GPIO_Init(GPIOD, &GPIO_InitStructureD);

	// GPIO_A.8 = LED
	GPIO_InitStructureA.GPIO_Pin   = GPIO_Pin_8;
	GPIO_InitStructureA.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructureA.GPIO_Mode  = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructureA);

	GPIO_SetBits(GPIOA , GPIO_Pin_8);	// LED_.
}


/*******************************************************************************
* Function Name  : GPIO_Configuration
* Description    : Configures the different GPIO ports.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void GPIO_Configuration_CQ_STARM(void)
{
//	GPIO_InitTypeDef GPIO_InitStructureA;
	GPIO_InitTypeDef GPIO_InitStructureB;
	GPIO_InitTypeDef GPIO_InitStructureC;
//	GPIO_InitTypeDef GPIO_InitStructureD;

	// GPIO A,B,C,D ɃNbN^.
	RCC_APB2PeriphClockCmd(
	    RCC_APB2Periph_GPIOA |
	    RCC_APB2Periph_GPIOB |
	    RCC_APB2Periph_GPIOC | /* == RCC_APB2Periph_GPIO_IOAIN      */
	    RCC_APB2Periph_GPIOD   /* == RCC_APB2Periph_GPIO_DISCONNECT */
	    , ENABLE);

	// GPIO_B
	GPIO_InitStructureB.GPIO_Pin   = 0;
	GPIO_InitStructureB.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructureB.GPIO_Mode  = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOB, &GPIO_InitStructureB);

	// GPIO_C.6 = LED
	GPIO_InitStructureC.GPIO_Pin   = GPIO_Pin_6;
	GPIO_InitStructureC.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructureC.GPIO_Mode  = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOC, &GPIO_InitStructureC);

	GPIO_SetBits(GPIOC , GPIO_Pin_6);	// LED_.
}

/*******************************************************************************
* Function Name  : GPIO_Configuration
* Description    : Configures the different GPIO ports.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void GPIO_Configuration_STBEE(void)
{
//	GPIO_InitTypeDef GPIO_InitStructureA;
	GPIO_InitTypeDef GPIO_InitStructureB;
	GPIO_InitTypeDef GPIO_InitStructureC;
	GPIO_InitTypeDef GPIO_InitStructureD;

	// GPIO A,B,C,D ɃNbN^.
	RCC_APB2PeriphClockCmd(
	    RCC_APB2Periph_GPIOA |
	    RCC_APB2Periph_GPIOB |
	    RCC_APB2Periph_GPIOC | /* == RCC_APB2Periph_GPIO_IOAIN      */
	    RCC_APB2Periph_GPIOD   /* == RCC_APB2Periph_GPIO_DISCONNECT */
	    , ENABLE);

	// GPIO_B
	GPIO_InitStructureB.GPIO_Pin   = 0;
	GPIO_InitStructureB.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructureB.GPIO_Mode  = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOB, &GPIO_InitStructureB);

	/* PD.3 used as USB pull-up (LOW=pull-up) */
	/* PD.4 = LED */
	GPIO_InitStructureD.GPIO_Pin   = GPIO_Pin_3|GPIO_Pin_4;	// PD3  USB PullUp炵.
	GPIO_InitStructureD.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructureD.GPIO_Mode  = GPIO_Mode_Out_OD;
	GPIO_Init(GPIOD, &GPIO_InitStructureD);

	// GPIO_C.6 = LED
	GPIO_InitStructureC.GPIO_Pin   = GPIO_Pin_6;
	GPIO_InitStructureC.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructureC.GPIO_Mode  = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOC, &GPIO_InitStructureC);

}
/*******************************************************************************
* Function Name  : GPIO_Configuration
* Description    : Configures the different GPIO ports.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void GPIO_Configuration_STBEE_MINI(void)
{
	GPIO_InitTypeDef GPIO_InitStructureA;
	GPIO_InitTypeDef GPIO_InitStructureB;
	GPIO_InitTypeDef GPIO_InitStructureC;
//	GPIO_InitTypeDef GPIO_InitStructureD;

	// JTAG𖳌ɂ܂B
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO , ENABLE);
	AFIO->MAPR = _BV(26);

	// GPIO A,B,C,D ɃNbN^.
	RCC_APB2PeriphClockCmd(
	    RCC_APB2Periph_GPIOA |
	    RCC_APB2Periph_GPIOB |
	    RCC_APB2Periph_GPIOC | /* == RCC_APB2Periph_GPIO_IOAIN      */
	    RCC_APB2Periph_GPIOD   /* == RCC_APB2Periph_GPIO_DISCONNECT */
	    , ENABLE);

	// GPIO_A 13,15=LED 14=USB_Pullup 9=USART1_Tx
	GPIO_InitStructureA.GPIO_Pin   = GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15 | GPIO_Pin_9;
	GPIO_InitStructureA.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructureA.GPIO_Mode  = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructureA);

	// GPIO_B
	GPIO_InitStructureB.GPIO_Pin   = 0;
	GPIO_InitStructureB.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructureB.GPIO_Mode  = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOB, &GPIO_InitStructureB);

	// GPIO_C.6 = LED
	GPIO_InitStructureC.GPIO_Pin   = GPIO_Pin_6;
	GPIO_InitStructureC.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructureC.GPIO_Mode  = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOC, &GPIO_InitStructureC);

}


/********************************************************************
 *
 ********************************************************************
 */
void GPIO_Configuration(void)
{
	switch (hwtype) {
	default:
	case HW_STM8S:
		GPIO_Configuration_STM8S_D();
		break;
	case HW_STBEE:
		GPIO_Configuration_STBEE();
		break;
	case HW_CQ_STARM:
		GPIO_Configuration_CQ_STARM();
		break;
	case HW_STBEE_MINI:
		GPIO_Configuration_STBEE_MINI();
		break;
	}
}


//	ȉpullupPIN_(LŃANeBu).
//		CQ_STARM
//		STBEE

//	ȉ́ApullupPINgĂȂ.
//		STM8_D
//
void USB_Cable_Config(FunctionalState NewState)
{
	void* usb_pullup_port = usb_port_tab[hwtype];
	int   usb_pullup_bit  = usb_pin_tab[hwtype];
	int rev=0;

	if (hwtype==HW_STBEE_MINI) {
		rev=1;    // pull-uptɐ.
	}

	if (usb_pullup_port == 0) return;	// pull-upoȂ.

	if (rev) {
		if (NewState == DISABLE) {
			GPIO_ResetBits(usb_pullup_port,usb_pullup_bit);
		} else {
			GPIO_SetBits(  usb_pullup_port,usb_pullup_bit);
		}
	} else {
		if (NewState != DISABLE) {
			GPIO_ResetBits(usb_pullup_port,usb_pullup_bit);
		} else {
			GPIO_SetBits(  usb_pullup_port,usb_pullup_bit);
		}
	}
}


/********************************************************************
 *
 ********************************************************************
 */

/********************************************************************
 *
 ********************************************************************
 */
void wait_us(int us);
/********************************************************************
 *
 ********************************************************************
 */
void wait_ms(int ms)
{
	int i;
	for (i=0; i<ms; i++) {
		wait_us(1000*8);
	}
}

/********************************************************************
 *
 ********************************************************************
 */
void wait_us(int us)
{
	int i;
	for (i=0; i<us; i++) {
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
	}
}


